@@ -12,23 +12,25 @@
 #include <orc/orcmmx.h>
 #include <orc/orcutils.h>
 #include <orc/orcdebug.h>
+#include <orc/orcinternal.h>
 
 #define MMX 1
+#ifdef MMX
+#  define ORC_REG_SIZE 8
+#else
+#  define ORC_REG_SIZE 16
+#endif
 #define SIZE 65536
 
 #define ORC_MMX_ALIGNED_DEST_CUTOFF 64
 
-void orc_mmx_emit_loop (OrcCompiler *compiler, int offset, int update);
+static void orc_mmx_emit_loop (OrcCompiler *compiler, int offset, int update);
 
-void orc_compiler_mmx_init (OrcCompiler *compiler);
-unsigned int orc_compiler_mmx_get_default_flags (void);
-void orc_compiler_mmx_assemble (OrcCompiler *compiler);
 void orc_compiler_mmx_register_rules (OrcTarget *target);
-void orc_mmx_emit_invariants (OrcCompiler *compiler);
+static void orc_compiler_mmx_init (OrcCompiler *compiler);
+static unsigned int orc_compiler_mmx_get_default_flags (void);
+static void orc_compiler_mmx_assemble (OrcCompiler *compiler);
 
-
-void orc_compiler_rewrite_vars (OrcCompiler *compiler);
-void orc_compiler_dump (OrcCompiler *compiler);
 void mmx_load_constant (OrcCompiler *compiler, int reg, int size, int value);
 void mmx_load_constant_long (OrcCompiler *compiler, int reg,
     OrcConstant *constant);
@@ -83,12 +85,12 @@ orc_mmx_init (void)
   orc_compiler_mmx_register_rules (&mmx_target);
 }
 
-unsigned int
+static unsigned int
 orc_compiler_mmx_get_default_flags (void)
 {
   unsigned int flags = 0;
 
-#if defined (HAVE_AMD64)
+#if defined(HAVE_AMD64)
   flags |= ORC_TARGET_MMX_64BIT;
 #endif
   if (_orc_compiler_flag_debug) {
@@ -135,7 +137,7 @@ mmx_get_flag_name (int shift)
   return NULL;
 }
 
-void
+static void
 orc_compiler_mmx_init (OrcCompiler *compiler)
 {
   int i;
@@ -156,25 +158,19 @@ orc_compiler_mmx_init (OrcCompiler *compiler)
       compiler->valid_regs[i] = 1;
     }
     compiler->valid_regs[X86_ESP] = 0;
-#ifndef MMX
-    for(i=X86_MM0;i<X86_MM0+16;i++){
+    for(i=X86_MM0;i<X86_MM0+ORC_REG_SIZE;i++){
       compiler->valid_regs[i] = 1;
     }
-#else
-    for(i=X86_MM0;i<X86_MM0+8;i++){
-      compiler->valid_regs[i] = 1;
-    }
-#endif
     compiler->save_regs[X86_EBX] = 1;
     compiler->save_regs[X86_EBP] = 1;
     compiler->save_regs[X86_R12] = 1;
     compiler->save_regs[X86_R13] = 1;
     compiler->save_regs[X86_R14] = 1;
     compiler->save_regs[X86_R15] = 1;
-#if defined(HAVE_OS_WIN32) || defined(HAVE_OS_CYGWIN)
+#if defined(HAVE_OS_WIN32) || defined(__CYGWIN__)
     compiler->save_regs[X86_EDI] = 1;
     compiler->save_regs[X86_ESI] = 1;
-    for(i=X86_MM0+6;i<X86_MM0+16;i++){
+    for(i=X86_MM0+6;i<X86_MM0+ORC_REG_SIZE;i++){
       compiler->save_regs[i] = 1;
     }
 #endif
@@ -199,7 +195,7 @@ orc_compiler_mmx_init (OrcCompiler *compiler)
   }
 
   if (compiler->is_64bit) {
-#if defined(HAVE_OS_WIN32) || defined(HAVE_OS_CYGWIN)
+#if defined(HAVE_OS_WIN32) || defined(__CYGWIN__)
     compiler->exec_reg = X86_ECX;
     compiler->gp_tmpreg = X86_EDX;
 #else
@@ -456,7 +452,7 @@ mmx_load_constant_long (OrcCompiler *compiler, int reg,
     orc_x86_emit_mov_reg_memoffset (compiler, 4, compiler->gp_tmpreg,
         offset + 4*i, compiler->exec_reg);
   }
-  orc_x86_emit_mov_memoffset_mmx (compiler, 16, offset, compiler->exec_reg,
+  orc_x86_emit_mov_memoffset_mmx (compiler, ORC_REG_SIZE, offset, compiler->exec_reg,
       reg, FALSE);
 
 }
@@ -487,7 +483,7 @@ mmx_load_constants_outer (OrcCompiler *compiler)
     }
   }
 
-  orc_mmx_emit_invariants (compiler);
+  orc_compiler_emit_invariants (compiler);
 
   /* FIXME move to a better place */
   for(i=0;i<compiler->n_constants;i++){
@@ -742,20 +738,6 @@ orc_emit_split_2_regions (OrcCompiler *compiler)
       (int)ORC_STRUCT_OFFSET(OrcExecutor,counter3), compiler->exec_reg);
 }
 
-#ifndef MMX
-static int
-orc_program_has_float (OrcCompiler *compiler)
-{
-  int j;
-  for(j=0;j<compiler->n_insns;j++){
-    OrcInstruction *insn = compiler->insns + j;
-    OrcStaticOpcode *opcode = insn->opcode;
-    if (opcode->flags & ORC_STATIC_OPCODE_FLOAT) return TRUE;
-  }
-  return FALSE;
-}
-#endif
-
 #define LABEL_REGION1_SKIP 1
 #define LABEL_INNER_LOOP_START 2
 #define LABEL_REGION2_SKIP 3
@@ -764,8 +746,51 @@ orc_program_has_float (OrcCompiler *compiler)
 #define LABEL_STEP_DOWN(x) (8+(x))
 #define LABEL_STEP_UP(x) (13+(x))
 
+static void
+orc_compiler_mmx_save_registers (OrcCompiler *compiler)
+{
+  int i;
+  int saved = 0;
+  for (i = 0; i < ORC_REG_SIZE; ++i) {
+    if (compiler->save_regs[X86_MM0 + i] == 1) {
+      ++saved;
+    }
+  }
+  if (saved > 0) {
+    orc_x86_emit_mov_imm_reg (compiler, 4, ORC_REG_SIZE * saved, compiler->gp_tmpreg);
+    orc_x86_emit_sub_reg_reg (compiler, compiler->is_64bit ? 8 : 4,
+        compiler->gp_tmpreg, X86_ESP);
+    saved = 0;
+    for (i = 0; i < ORC_REG_SIZE; ++i) {
+      if (compiler->save_regs[X86_MM0 + i] == 1) {
+        orc_x86_emit_mov_mmx_memoffset (compiler, ORC_REG_SIZE, X86_MM0 + i,
+            saved * ORC_REG_SIZE, X86_ESP, FALSE, FALSE);
+        ++saved;
+      }
+    }
+  }
+}
 
-void
+static void
+orc_compiler_mmx_restore_registers (OrcCompiler *compiler)
+{
+  int i;
+  int saved = 0;
+  for (i = 0; i < ORC_REG_SIZE; ++i) {
+    if (compiler->save_regs[X86_MM0 + i] == 1) {
+      orc_x86_emit_mov_memoffset_mmx (compiler, ORC_REG_SIZE, saved * ORC_REG_SIZE, X86_ESP,
+          X86_MM0 + i, FALSE);
+      ++saved;
+    }
+  }
+  if (saved > 0) {
+    orc_x86_emit_mov_imm_reg (compiler, 4, ORC_REG_SIZE * saved, compiler->gp_tmpreg);
+    orc_x86_emit_add_reg_reg (compiler, compiler->is_64bit ? 8 : 4,
+        compiler->gp_tmpreg, X86_ESP);
+  }
+}
+
+static void
 orc_compiler_mmx_assemble (OrcCompiler *compiler)
 {
 #ifndef MMX
@@ -804,6 +829,8 @@ orc_compiler_mmx_assemble (OrcCompiler *compiler)
 
   orc_x86_emit_prologue (compiler);
 
+  orc_compiler_mmx_save_registers (compiler);
+
 #ifndef MMX
   if (orc_program_has_float (compiler)) {
     set_mxcsr = TRUE;
@@ -990,6 +1017,9 @@ orc_compiler_mmx_assemble (OrcCompiler *compiler)
 #else
   orc_x86_emit_emms (compiler);
 #endif
+
+  orc_compiler_mmx_restore_registers (compiler);
+
   orc_x86_emit_epilogue (compiler);
 
   orc_x86_calculate_offsets (compiler);
@@ -998,7 +1028,7 @@ orc_compiler_mmx_assemble (OrcCompiler *compiler)
   orc_x86_do_fixups (compiler);
 }
 
-void
+static void
 orc_mmx_emit_loop (OrcCompiler *compiler, int offset, int update)
 {
   int j;
@@ -1029,19 +1059,6 @@ orc_mmx_emit_loop (OrcCompiler *compiler, int offset, int update)
 
     rule = insn->rule;
     if (rule && rule->emit) {
-      if (!(insn->opcode->flags & (ORC_STATIC_OPCODE_ACCUMULATOR|ORC_STATIC_OPCODE_LOAD|ORC_STATIC_OPCODE_STORE)) &&
-          compiler->vars[insn->dest_args[0]].alloc !=
-          compiler->vars[insn->src_args[0]].alloc) {
-#ifdef MMX
-        orc_mmx_emit_movq (compiler,
-            compiler->vars[insn->src_args[0]].alloc,
-            compiler->vars[insn->dest_args[0]].alloc);
-#else
-        orc_mmx_emit_movdqu (compiler,
-            compiler->vars[insn->src_args[0]].alloc,
-            compiler->vars[insn->dest_args[0]].alloc);
-#endif
-      }
       rule->emit (compiler, rule->emit_user, insn);
     } else {
       orc_compiler_error (compiler, "no code generation rule for %s",
@@ -1081,38 +1098,3 @@ orc_mmx_emit_loop (OrcCompiler *compiler, int offset, int update)
     }
   }
 }
-
-void
-orc_mmx_emit_invariants (OrcCompiler *compiler)
-{
-  int j;
-  OrcInstruction *insn;
-  OrcStaticOpcode *opcode;
-  OrcRule *rule;
-
-  for(j=0;j<compiler->n_insns;j++){
-    insn = compiler->insns + j;
-    opcode = insn->opcode;
-
-    if (!(insn->flags & ORC_INSN_FLAG_INVARIANT)) continue;
-
-    ORC_ASM_CODE(compiler,"# %d: %s\n", j, insn->opcode->name);
-
-    compiler->insn_shift = compiler->loop_shift;
-    if (insn->flags & ORC_INSTRUCTION_FLAG_X2) {
-      compiler->insn_shift += 1;
-    }
-    if (insn->flags & ORC_INSTRUCTION_FLAG_X4) {
-      compiler->insn_shift += 2;
-    }
-
-    rule = insn->rule;
-    if (rule && rule->emit) {
-      rule->emit (compiler, rule->emit_user, insn);
-    } else {
-      orc_compiler_error (compiler, "no code generation rule for %s",
-          opcode->name);
-    }
-  }
-}
-
