@@ -12,23 +12,25 @@
 #include <orc/orcsse.h>
 #include <orc/orcutils.h>
 #include <orc/orcdebug.h>
+#include <orc/orcinternal.h>
 
 #undef MMX
+#ifdef MMX
+#  define ORC_REG_SIZE 8
+#else
+#  define ORC_REG_SIZE 16
+#endif
 #define SIZE 65536
 
 #define ORC_SSE_ALIGNED_DEST_CUTOFF 64
 
-void orc_sse_emit_loop (OrcCompiler *compiler, int offset, int update);
+static void orc_sse_emit_loop (OrcCompiler *compiler, int offset, int update);
 
-void orc_compiler_sse_init (OrcCompiler *compiler);
-unsigned int orc_compiler_sse_get_default_flags (void);
-void orc_compiler_sse_assemble (OrcCompiler *compiler);
 void orc_compiler_sse_register_rules (OrcTarget *target);
-void orc_sse_emit_invariants (OrcCompiler *compiler);
+static void orc_compiler_sse_init (OrcCompiler *compiler);
+static unsigned int orc_compiler_sse_get_default_flags (void);
+static void orc_compiler_sse_assemble (OrcCompiler *compiler);
 
-
-void orc_compiler_rewrite_vars (OrcCompiler *compiler);
-void orc_compiler_dump (OrcCompiler *compiler);
 void sse_load_constant (OrcCompiler *compiler, int reg, int size, int value);
 void sse_load_constant_long (OrcCompiler *compiler, int reg,
     OrcConstant *constant);
@@ -83,7 +85,7 @@ orc_sse_init (void)
   orc_compiler_sse_register_rules (&sse_target);
 }
 
-unsigned int
+static unsigned int
 orc_compiler_sse_get_default_flags (void)
 {
   unsigned int flags = 0;
@@ -135,7 +137,7 @@ sse_get_flag_name (int shift)
   return NULL;
 }
 
-void
+static void
 orc_compiler_sse_init (OrcCompiler *compiler)
 {
   int i;
@@ -156,25 +158,19 @@ orc_compiler_sse_init (OrcCompiler *compiler)
       compiler->valid_regs[i] = 1;
     }
     compiler->valid_regs[X86_ESP] = 0;
-#ifndef MMX
-    for(i=X86_XMM0;i<X86_XMM0+16;i++){
+    for(i=X86_XMM0;i<X86_XMM0+ORC_REG_SIZE;i++){
       compiler->valid_regs[i] = 1;
     }
-#else
-    for(i=X86_XMM0;i<X86_XMM0+8;i++){
-      compiler->valid_regs[i] = 1;
-    }
-#endif
     compiler->save_regs[X86_EBX] = 1;
     compiler->save_regs[X86_EBP] = 1;
     compiler->save_regs[X86_R12] = 1;
     compiler->save_regs[X86_R13] = 1;
     compiler->save_regs[X86_R14] = 1;
     compiler->save_regs[X86_R15] = 1;
-#if defined(HAVE_OS_WIN32) || defined(HAVE_OS_CYGWIN)
+#if defined(HAVE_OS_WIN32) || defined(__CYGWIN__)
     compiler->save_regs[X86_EDI] = 1;
     compiler->save_regs[X86_ESI] = 1;
-    for(i=X86_XMM0+6;i<X86_XMM0+16;i++){
+    for(i=X86_XMM0+6;i<X86_XMM0+ORC_REG_SIZE;i++){
       compiler->save_regs[i] = 1;
     }
 #endif
@@ -199,7 +195,7 @@ orc_compiler_sse_init (OrcCompiler *compiler)
   }
 
   if (compiler->is_64bit) {
-#if defined(HAVE_OS_WIN32) || defined(HAVE_OS_CYGWIN)
+#if defined(HAVE_OS_WIN32) || defined(__CYGWIN__)
     compiler->exec_reg = X86_ECX;
     compiler->gp_tmpreg = X86_EDX;
 #else
@@ -456,7 +452,7 @@ sse_load_constant_long (OrcCompiler *compiler, int reg,
     orc_x86_emit_mov_reg_memoffset (compiler, 4, compiler->gp_tmpreg,
         offset + 4*i, compiler->exec_reg);
   }
-  orc_x86_emit_mov_memoffset_sse (compiler, 16, offset, compiler->exec_reg,
+  orc_x86_emit_mov_memoffset_sse (compiler, ORC_REG_SIZE, offset, compiler->exec_reg,
       reg, FALSE);
 
 }
@@ -487,7 +483,7 @@ sse_load_constants_outer (OrcCompiler *compiler)
     }
   }
 
-  orc_sse_emit_invariants (compiler);
+  orc_compiler_emit_invariants (compiler);
 
   /* FIXME move to a better place */
   for(i=0;i<compiler->n_constants;i++){
@@ -742,20 +738,6 @@ orc_emit_split_2_regions (OrcCompiler *compiler)
       (int)ORC_STRUCT_OFFSET(OrcExecutor,counter3), compiler->exec_reg);
 }
 
-#ifndef MMX
-static int
-orc_program_has_float (OrcCompiler *compiler)
-{
-  int j;
-  for(j=0;j<compiler->n_insns;j++){
-    OrcInstruction *insn = compiler->insns + j;
-    OrcStaticOpcode *opcode = insn->opcode;
-    if (opcode->flags & ORC_STATIC_OPCODE_FLOAT) return TRUE;
-  }
-  return FALSE;
-}
-#endif
-
 #define LABEL_REGION1_SKIP 1
 #define LABEL_INNER_LOOP_START 2
 #define LABEL_REGION2_SKIP 3
@@ -769,20 +751,20 @@ orc_compiler_sse_save_registers (OrcCompiler *compiler)
 {
   int i;
   int saved = 0;
-  for (i = 0; i < 16; ++i) {
+  for (i = 0; i < ORC_REG_SIZE; ++i) {
     if (compiler->save_regs[X86_XMM0 + i] == 1) {
       ++saved;
     }
   }
   if (saved > 0) {
-    orc_x86_emit_mov_imm_reg (compiler, 4, 16 * saved, compiler->gp_tmpreg);
+    orc_x86_emit_mov_imm_reg (compiler, 4, ORC_REG_SIZE * saved, compiler->gp_tmpreg);
     orc_x86_emit_sub_reg_reg (compiler, compiler->is_64bit ? 8 : 4,
         compiler->gp_tmpreg, X86_ESP);
     saved = 0;
-    for (i = 0; i < 16; ++i) {
+    for (i = 0; i < ORC_REG_SIZE; ++i) {
       if (compiler->save_regs[X86_XMM0 + i] == 1) {
-        orc_x86_emit_mov_sse_memoffset (compiler, 16, X86_XMM0 + i,
-            saved * 16, X86_ESP, FALSE, FALSE);
+        orc_x86_emit_mov_sse_memoffset (compiler, ORC_REG_SIZE, X86_XMM0 + i,
+            saved * ORC_REG_SIZE, X86_ESP, FALSE, FALSE);
         ++saved;
       }
     }
@@ -794,21 +776,21 @@ orc_compiler_sse_restore_registers (OrcCompiler *compiler)
 {
   int i;
   int saved = 0;
-  for (i = 0; i < 16; ++i) {
+  for (i = 0; i < ORC_REG_SIZE; ++i) {
     if (compiler->save_regs[X86_XMM0 + i] == 1) {
-      orc_x86_emit_mov_memoffset_sse (compiler, 16, saved * 16, X86_ESP,
+      orc_x86_emit_mov_memoffset_sse (compiler, ORC_REG_SIZE, saved * ORC_REG_SIZE, X86_ESP,
           X86_XMM0 + i, FALSE);
       ++saved;
     }
   }
   if (saved > 0) {
-    orc_x86_emit_mov_imm_reg (compiler, 4, 16 * saved, compiler->gp_tmpreg);
+    orc_x86_emit_mov_imm_reg (compiler, 4, ORC_REG_SIZE * saved, compiler->gp_tmpreg);
     orc_x86_emit_add_reg_reg (compiler, compiler->is_64bit ? 8 : 4,
         compiler->gp_tmpreg, X86_ESP);
   }
 }
 
-void
+static void
 orc_compiler_sse_assemble (OrcCompiler *compiler)
 {
 #ifndef MMX
@@ -1046,7 +1028,7 @@ orc_compiler_sse_assemble (OrcCompiler *compiler)
   orc_x86_do_fixups (compiler);
 }
 
-void
+static void
 orc_sse_emit_loop (OrcCompiler *compiler, int offset, int update)
 {
   int j;
@@ -1077,19 +1059,6 @@ orc_sse_emit_loop (OrcCompiler *compiler, int offset, int update)
 
     rule = insn->rule;
     if (rule && rule->emit) {
-      if (!(insn->opcode->flags & (ORC_STATIC_OPCODE_ACCUMULATOR|ORC_STATIC_OPCODE_LOAD|ORC_STATIC_OPCODE_STORE|ORC_STATIC_OPCODE_COPY)) &&
-          compiler->vars[insn->dest_args[0]].alloc !=
-          compiler->vars[insn->src_args[0]].alloc) {
-#ifdef MMX
-        orc_sse_emit_movq (compiler,
-            compiler->vars[insn->src_args[0]].alloc,
-            compiler->vars[insn->dest_args[0]].alloc);
-#else
-        orc_sse_emit_movdqu (compiler,
-            compiler->vars[insn->src_args[0]].alloc,
-            compiler->vars[insn->dest_args[0]].alloc);
-#endif
-      }
       rule->emit (compiler, rule->emit_user, insn);
     } else {
       orc_compiler_error (compiler, "no code generation rule for %s",
@@ -1129,38 +1098,3 @@ orc_sse_emit_loop (OrcCompiler *compiler, int offset, int update)
     }
   }
 }
-
-void
-orc_sse_emit_invariants (OrcCompiler *compiler)
-{
-  int j;
-  OrcInstruction *insn;
-  OrcStaticOpcode *opcode;
-  OrcRule *rule;
-
-  for(j=0;j<compiler->n_insns;j++){
-    insn = compiler->insns + j;
-    opcode = insn->opcode;
-
-    if (!(insn->flags & ORC_INSN_FLAG_INVARIANT)) continue;
-
-    ORC_ASM_CODE(compiler,"# %d: %s\n", j, insn->opcode->name);
-
-    compiler->insn_shift = compiler->loop_shift;
-    if (insn->flags & ORC_INSTRUCTION_FLAG_X2) {
-      compiler->insn_shift += 1;
-    }
-    if (insn->flags & ORC_INSTRUCTION_FLAG_X4) {
-      compiler->insn_shift += 2;
-    }
-
-    rule = insn->rule;
-    if (rule && rule->emit) {
-      rule->emit (compiler, rule->emit_user, insn);
-    } else {
-      orc_compiler_error (compiler, "no code generation rule for %s",
-          opcode->name);
-    }
-  }
-}
-
